@@ -114,6 +114,44 @@ int Manage_runstop(char *devname, int fd, int runstop)
 	return 0;
 }
 
+int Manage_resize(char *devname, int fd, long long size, int raid_disks)
+{
+	mdu_array_info_t info;
+	if (ioctl(fd, GET_ARRAY_INFO, &info) != 0) {
+		fprintf(stderr, Name ": Cannot get array information for %s: %s\n",
+			devname, strerror(errno));
+		return 1;
+	}
+	if (size >= 0)
+		info.size = size;
+	if (raid_disks > 0)
+		info.raid_disks = raid_disks;
+	if (ioctl(fd, SET_ARRAY_INFO, &info) != 0) {
+		fprintf(stderr, Name ": Cannot set device size/shape for %s: %s\n",
+			devname, strerror(errno));
+		return 1;
+	}
+	return 0;
+}
+
+int Manage_reconfig(char *devname, int fd, int layout)
+{
+	mdu_array_info_t info;
+	if (ioctl(fd, GET_ARRAY_INFO, &info) != 0) {
+		fprintf(stderr, Name ": Cannot get array information for %s: %s\n",
+			devname, strerror(errno));
+		return 1;
+	}
+	info.layout = layout;
+	printf("layout set to %d\n", info.layout);
+	if (ioctl(fd, SET_ARRAY_INFO, &info) != 0) {
+		fprintf(stderr, Name ": Cannot set layout for %s: %s\n",
+			devname, strerror(errno));
+		return 1;
+	}
+	return 0;
+}
+
 int Manage_subdevs(char *devname, int fd,
 		   mddev_dev_t devlist)
 {
@@ -177,6 +215,7 @@ int Manage_subdevs(char *devname, int fd,
 			 * in case
 			 */
 			for (j=0; j<array.nr_disks; j++) {
+				disc.number = j;
 				if (ioctl(fd, GET_DISK_INFO, &disc))
 					break;
 				if (disc.major==0 && disc.minor==0)
