@@ -47,7 +47,7 @@ static char *percentalerts[] = {
 int Monitor(mddev_dev_t devlist,
 	    char *mailaddr, char *alert_cmd,
 	    int period, int daemonise, int scan, int oneshot,
-	    char *config, int test)
+	    char *config, int test, char* pidfile)
 {
 	/*
 	 * Every few seconds, scan every md device looking for changes
@@ -127,7 +127,18 @@ int Monitor(mddev_dev_t devlist,
 	if (daemonise) {
 		int pid = fork();
 		if (pid > 0) {
-			printf("%d\n", pid);
+			if (!pidfile)
+				printf("%d\n", pid);
+			else {
+				FILE *pid_file;
+				pid_file=fopen(pidfile, "w");
+				if (!pid_file)
+					perror("cannot create pid file");
+				else {
+					fprintf(pid_file,"%d\n", pid);
+					fclose(pid_file);
+				}
+			}
 			return 0;
 		}
 		if (pid < 0) {
@@ -163,6 +174,7 @@ int Monitor(mddev_dev_t devlist,
 	} else {
 		mddev_dev_t dv;
 		for (dv=devlist ; dv; dv=dv->next) {
+			mddev_ident_t mdlist = conf_get_ident(config, dv->devname);
 			struct state *st = malloc(sizeof *st);
 			if (st == NULL)
 				continue;
@@ -174,6 +186,11 @@ int Monitor(mddev_dev_t devlist,
 			st->percent = -2;
 			st->expected_spares = -1;
 			st->spare_group = NULL;
+			if (mdlist) {
+				st->expected_spares = mdlist->spare_disks;
+				if (mdlist->spare_group)
+					st->spare_group = strdup(mdlist->spare_group);
+			}
 			statelist = st;
 		}
 	}
@@ -216,7 +233,8 @@ int Monitor(mddev_dev_t devlist,
 				close(fd);
 				continue;
 			}
-			if (array.level != 1 && array.level != 5 && array.level != -4) {
+			if (array.level != 1 && array.level != 5 && array.level != -4 &&
+				array.level != 6 && array.level != 10) {
 				if (!st->err)
 					alert("DeviceDisappeared", dev, "Wrong-Level",
 					      mailaddr, alert_cmd);
@@ -421,6 +439,8 @@ int Monitor(mddev_dev_t devlist,
 		}
 		test = 0;
 	}
+	if (pidfile)
+		unlink(pidfile);
 	return 0;
 }
 
